/***************************************************************************
 *   Copyright Brian Ledbetter 2001-2003 <brian@shadowcom.net>             *
 *   Copyright Ravikiran Rajagopal 2003 <ravi@kde.org>                     *
 *   Copyright Laurent Montel 2004,2005 <lmontel@mandriva.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License (version 2) as   *
 *   published by the Free Software Foundation. (The original KSplash/ML   *
 *   codebase (upto version 0.95.3) is BSD-licensed.)                      *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kgenericfactory.h>
#include <kglobalsettings.h>
#include <klocale.h>
#include <kstandarddirs.h>

#include <qcheckbox.h>
#include <qdesktopwidget.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qwidget.h>

#include "objkstheme.h"
#include "themeengine.h"
#include "mdktheme.h"
#include "mdktheme.moc"

DefaultConfig::DefaultConfig( QWidget *parent, KConfig *config )
    :ThemeEngineConfig( parent, config )
{
  mConfig->setGroup( QString("KSplash Theme: Mdk") );
  QVBox *hbox = new QVBox( this );
  mFlash = new QCheckBox( i18n("Icons flash while they are starting"), hbox );
  mFlash->setChecked( mConfig->readBoolEntry("Icons Flashing",true) );
  mAlwaysShow = new QCheckBox( i18n("Always show progress bar"), hbox );
  mAlwaysShow->setChecked( mConfig->readBoolEntry("Always Show Progress",true) );
}

void DefaultConfig::save()
{
  kdDebug() << "DefaultConfig::save()" << endl;
  mConfig->setGroup( QString("KSplash Theme: Mdk") );
  mConfig->writeEntry( "Icons Flashing", mFlash->isChecked() );
  mConfig->writeEntry( "Always Show Progress", mAlwaysShow->isChecked() );
  mConfig->sync();
}
#define BIDI 1

Thememdk::Thememdk( QWidget *parent, const char *name, const QStringList &args )
    :ThemeEngine( parent, name, args )
{

  mActivePixmap = mInactivePixmap = 0L;
  mState = 0;

  _readSettings();
  _initUi();

  if( mIconsFlashing )
  {
    mFlashTimer = new QTimer( this );
    connect( mFlashTimer, SIGNAL(timeout()), this, SLOT(flash()) );
    mFlashPixmap1 = new QPixmap();
    mFlashPixmap2 = new QPixmap();

  }
  else
  {
    mFlashTimer = 0L;
    mFlashPixmap1 = 0L;
    mFlashPixmap2 = 0L;
  }
}

Thememdk::~Thememdk()
{
    delete mFlashPixmap1;
    delete mFlashPixmap2;
}

void Thememdk::_initUi()
{
  QString resource_prefix;

  QVBox *vbox = new QVBox( this );
  vbox->setBackgroundMode(NoBackground);


  QString activePix, inactivePix;
#if BIDI
  if ( QApplication::reverseLayout() )
  {
      activePix = _findPicture(QString("mdk-splash_active_bar_bidi.png"));
      inactivePix = _findPicture(QString("mdk-splash_inactive_bar_bidi.png"));
  }
  else
#endif
  {
      activePix = _findPicture(QString("mdk-splash_active_bar.png"));
      inactivePix = _findPicture(QString("mdk-splash_inactive_bar.png"));
  }
  //kdDebug() << "Inactive pixmap: " << inactivePix << endl;
  //kdDebug() << "Active pixmap:   " <<   activePix << endl;

  mActivePixmap = new QPixmap( activePix );
  mInactivePixmap = new QPixmap( inactivePix );

  if (mActivePixmap->isNull())
  {
    mActivePixmap->resize(200,100);
    mActivePixmap->fill(Qt::blue);
  }
  if (mInactivePixmap->isNull())
  {
    mInactivePixmap->resize(200,100);
    mInactivePixmap->fill(Qt::black);
  }

  QPixmap tlimage( _findPicture(QString("mdk-splash_top.png")) );
  if (tlimage.isNull())
  {
    tlimage.resize(200,100);
    tlimage.fill(Qt::blue);
  }
  QLabel *top_label = new QLabel( vbox );
  top_label->setPixmap( tlimage );
  top_label->setFixedSize( tlimage.width(), tlimage.height() );
  top_label->setBackgroundMode(NoBackground);

  mBarLabel = new QLabel( vbox );
  mBarLabel->setPixmap(*mInactivePixmap);
  mBarLabel->setBackgroundMode(NoBackground);

  QPixmap blimage( _findPicture(QString("mdk-splash_bottom.png")) );
  if (blimage.isNull())
  {
    blimage.resize(200,100);
    blimage.fill(Qt::black);
  }
  QLabel *bottom_label = new QLabel( vbox );
  bottom_label->setPaletteBackgroundPixmap( blimage );


      mLabel = new QLabel( bottom_label );
      mLabel->setBackgroundOrigin( QWidget::ParentOrigin );
      mLabel->setPaletteForegroundColor( mLabelForeground );
      mLabel->setPaletteBackgroundPixmap( blimage );
      QFont f(mLabel->font());
      f.setBold(TRUE);
      mLabel->setFont(f);

      mProgressBar = new KProgress( bottom_label );

      bottom_label->setFixedWidth( QMAX(blimage.width(),tlimage.width()) );
      bottom_label->setFixedHeight( mLabel->sizeHint().height()+4 );

      // 3 pixels of whitespace between the label and the progressbar.
      mLabel->resize( bottom_label->width() - 120 - 7, bottom_label->height() );

      mProgressBar->setFixedSize( 120, mLabel->height() );
      
      if (QApplication::reverseLayout()){
            mProgressBar->move( 2, 0 );   
	    mLabel->move( mProgressBar->width() + 4, 0);
      }
      else{
            mProgressBar->move( bottom_label->width() - mProgressBar->width() - 4, 0);
	    mLabel->move( 2, 0 );
      }
      
      mProgressBar->hide();



  setFixedWidth( mInactivePixmap->width() );
  setFixedHeight( mInactivePixmap->height() +
                  top_label->height() + bottom_label->height() );

  const QRect rect = kapp->desktop()->screenGeometry( mTheme->xineramaScreen() );
  // KGlobalSettings::splashScreenDesktopGeometry(); cannot be used here.
  // kdDebug() << "Thememdk::_initUi" << rect << endl;

  move( rect.x() + (rect.width() - size().width())/2,
        rect.y() + (rect.height() - size().height())/2 );
}

// Attempt to find overrides elsewhere?
void Thememdk::_readSettings()
{
  if( !mTheme )
    return;

  KConfig *cfg = mTheme->themeConfig();
  if( !cfg )
    return;

  cfg->setGroup( QString("KSplash Theme: %1").arg(mTheme->theme()) );

  mIconsFlashing = cfg->readBoolEntry( "Icons Flashing", true );
  QColor df(Qt::white);
  mLabelForeground = cfg->readColorEntry( "Label Foreground", &df );
}

/*
 * Thememdk::slotUpdateState(): IF in Default mode, THEN adjust the bar
 * pixmap label. Whee, phun!
 *
 * A similar method exists in the old KSplash.
 */
void Thememdk::slotUpdateState()
{

    if( !mState )
    {
        mState++;
        return;
    }

    if( mState > 8 )
        mState = 8;

    if( mIconsFlashing )
    {

        *mFlashPixmap1 = updateBarPixmap( mState );
        *mFlashPixmap2 = updateBarPixmap( mState+1 );
        mBarLabel->setPixmap(*mFlashPixmap2);
        mFlashTimer->stop();

        if( mState < 8 )
            mFlashTimer->start(400);
    }
    else
        mBarLabel->setPixmap( updateBarPixmap( mState ) );

    mState++;
}

/*
 * Thememdk::updateBarPixmap(): IF in Default mode, THEN adjust the
 * bar pixmap to reflect the current state. WARNING! KSplash Default
 * does NOT support our "Restoring Session..." state. We will need
 * to reflect that somehow.
 */
QPixmap Thememdk::updateBarPixmap( int state )
{
    int offs = 0;

    QPixmap x;
    if( !mActivePixmap ) return( x );
#if BIDI
    if ( state > 7 ) return (  x );
#endif

    offs = state * 58;
    if ( state == 2 )
        offs += 2;
    else if ( state == 3 )
        offs += 15;
    else if ( state == 4 )
        offs += 15;
    else if ( state == 5 )
        offs += 22;
    else if ( state == 6 )
        offs += 33;
    else if ( state == 7 )
        offs += 41;

    QPixmap tmp(*mActivePixmap);
    QPainter p(&tmp);
#if BIDI
    if ( QApplication::reverseLayout() )
        p.drawPixmap(0, 0, *mInactivePixmap, 0, 0, tmp.width()-offs );
    else
#endif
        p.drawPixmap(offs, 0, *mInactivePixmap, offs, 0);
    return tmp ;
}

void Thememdk::flash()
{
  if( !mIconsFlashing )
    return;
  QPixmap *swap = mFlashPixmap1;
  mFlashPixmap1 = mFlashPixmap2;
  mFlashPixmap2 = swap;
  mBarLabel->setPixmap(*mFlashPixmap2);
}

QString Thememdk::_findPicture( const QString &pic )
{
  // Don't use ObjKsTheme::locateThemeData here for compatibility reasons.
  QString f = pic;
  if (mTheme->loColor())
    f = QString("locolor/")+f;
  //kdDebug() << "Searching for " << f << endl;
  //kdDebug() << "Theme directory: " << mTheme->themeDir() << endl;
  //kdDebug() << "Theme name:      " << mTheme->theme() << endl;
  QString p = QString::null;
  if ((p = locate("appdata",mTheme->themeDir()+f)).isEmpty())
    if ((p = locate("appdata",mTheme->themeDir()+"pics/"+f)).isEmpty())
      if ((p = locate("appdata", QString("pics/")+mTheme->theme()+"/"+f)).isEmpty())
        if ((p = locate("appdata",f)).isEmpty())
          if ((p = locate("appdata",QString("pics/")+f)).isEmpty())
            if ((p = locate("data",QString("pics/")+f)).isEmpty())
              ; // No more places to search
  return p;
}
