/***************************************************************************
 *   Copyright (C) 2006 by Helio Chissini de Castro   *
 *   helio@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <qlayout.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <qdir.h>
#include <qfile.h>

#include <klocale.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kparts/genericfactory.h>

#include "profilechooser.h"

typedef KGenericFactory<profilechooser, QWidget> profilechooserFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_profilechooser, profilechooserFactory("kcmprofilechooser"))
	
profilechooser::profilechooser(QWidget *parent, const char *name, const QStringList&)
    : KCModule(parent, name), myAboutData(0)
{
	QGridLayout *mainLayout = new QGridLayout( this, 1, 1, 11, 6, "MainLayout");
	
	defaultLabel = new QLabel( this );
	mainLayout->addWidget( defaultLabel, 0, 0 );
	
	availableProfiles = new QListBox( this, "availableProfiles" );
	mainLayout->addWidget( availableProfiles, 1, 0 );
	
   QHBoxLayout *h = new QHBoxLayout( 0, 0, 2, "ButLayout" );
   KPushButton *bt = new KPushButton( i18n( "Default" ), this, "DefaultProfileButton" );
   connect( bt, SIGNAL( clicked() ), SLOT( orderChange() ) );
   h->addWidget( bt );
   
   bt = new KPushButton( i18n( "Add" ), this, "AddProfileButton" );	
   connect( bt, SIGNAL( clicked() ), SLOT( addProfile() ) );
   h->addWidget( bt );
   
   bt = new KPushButton( i18n( "Remove" ), this, "RemoveProfileButton" );
   connect( bt, SIGNAL( clicked() ), SLOT( removeProfile() ) );
   h->addWidget( bt );
   
   mainLayout->addLayout( h, 2, 0 );
   
   profileList.clear();
 
   setButtons( Apply|Default );
   
   setQuickHelp( i18n("<h1>Mandriva KDE Profile Chooser</h1>"
   	"Selects specific profiles to customize kde without touching main KDE configuration."
   	"Profiles are identified by a file calles \"mdvprofile\" in their root trees" ) ); 
   
   load();
};


profilechooser::~profilechooser()
{
}


void profilechooser::load()
{
	bool first=true;
	KConfig *cfg = new KConfig("/etc/kderc");
	cfg->setGroup("Directories-default");
	QStringList current = cfg->readListEntry( "prefixes" );
	delete cfg;
	QStringList::Iterator it = current.begin();
	while( it != current.end())
	{
		if ( putDescription( *it, first ) )
		{
			
			first=false;
		}
		it++;
    }
}


void profilechooser::save()
{
	KConfig *cfg = new KConfig("/etc/kderc");
	cfg->setGroup("Directories-default");
	
	QStringList val = profileList.keys();
	val.remove( defaultProfile );
	val.prepend( defaultProfile );
	cfg->writeEntry( "prefixes", val );
	cfg->sync();
	delete cfg;
	emit changed(true);
}

void profilechooser::configChanged()
{
    // insert your saving code here...
    emit changed(true);
}

void profilechooser::addProfile()
{
	KURL fdg = KFileDialog::getOpenURL( ":<keyword>", "mdvprofile", this, i18n( "Locate Profile Dir" ) );
	
	if( fdg.path() != QString::null )
	{
		if( profileList.contains( fdg.directory() ) )
		{
			KMessageBox::information( this, i18n( "Profile already present on list" ) );
		}
		else
		{
			putDescription( fdg.directory() );
			emit changed(true);
		} 
	}
}

bool profilechooser::putDescription( QString filename, bool first )
{
	if ( QFile::exists( filename ) )
	{
		QFile file( filename + "/mdvprofile" );
		if ( file.open( IO_ReadOnly ) )
		{
			QTextStream stream( &file );
			QString l = stream.readLine();
			profileList.insert( filename, l );
			availableProfiles->insertItem( l );
			file.close();
			if ( first || availableProfiles->count() == 1 )
			{
				defaultProfile = filename;
				defaultLabel->setText( i18n("Default") + ": <b>" + l + "</b>" );
			}
			file.close();
			return true;
		}
	}
	return false;
}

void profilechooser::removeProfile()
{
	int item = availableProfiles->currentItem();
	if( item != -1 )
	{
		int id = profileList.values().findIndex( availableProfiles->currentText() );
		availableProfiles->removeItem( item );
		profileList.remove( profileList.keys()[id] );
		if( item == 0 )
		{
			defaultProfile = profileList.keys()[0];
			defaultLabel->setText( i18n("Default") + ": <b>" + profileList.values()[0]  + "</b>" );
		}
		
		emit changed(true);
	}
}

void profilechooser::orderChange()
{
	int item = availableProfiles->currentItem();
	if( item != -1 )
	{
		int id = profileList.values().findIndex( availableProfiles->currentText() );
		defaultProfile = profileList.keys()[id];
		defaultLabel->setText( i18n("Default") + ": <b>" +  availableProfiles->currentText() + "</b>" );
		emit changed(true);	
	}
}

#include "profilechooser.moc"
